@@ -46,17 +46,18 @@ def mapstar(args):
 #
 
 class Introspector(threading.Thread):
-    def __init__(self, conn, worker_thread_id, *args, **kwargs):
+    def __init__(self, conn, worker_thread_id, state, *args, **kwargs):
         threading.Thread.__init__(self, *args, **kwargs)
         self._conn = conn
         self.worker_thread_id = worker_thread_id
+        self.state = state
         
     def run(self):
 
         while True:
             self._conn.recv()
-            self._conn.send(self.tracebacks())
-    def tracebacks(self):
+            self._conn.send(self.introspect())
+    def introspect(self):
         import traceback
         current_thread = threading.current_thread().ident
         try:
@@ -67,7 +68,7 @@ class Introspector(threading.Thread):
                 if id != self.worker_thread_id:
                     continue
                 header = 'Worker (pid: %d)\n' % os.getpid()
-                return header + ''.join(traceback.format_stack(stack))
+                return { 'state' : self.state['STATE'], 'trace' : header + ''.join(traceback.format_stack(stack)) }
         finally:
             del stack
 
@@ -79,7 +80,11 @@ class Introspector(threading.Thread):
 def worker(conn, inqueue, outqueue, initializer=None, initargs=(), maxtasks=None):
     assert maxtasks is None or (type(maxtasks) == int and maxtasks > 0)
     
-    introspector = Introspector(conn, threading.current_thread().ident)
+    state = {
+        'STATE' : 'INIT',
+    }
+    
+    introspector = Introspector(conn, threading.current_thread().ident, state)
     introspector.daemon = True
     introspector.start()
     
@@ -95,6 +100,7 @@ def worker(conn, inqueue, outqueue, initializer=None, initargs=(), maxtasks=None
     completed = 0
     while maxtasks is None or (maxtasks and completed < maxtasks):
         try:
+            state['STATE'] = 'WAITING'
             task = get()
         except (EOFError, IOError):
             debug('worker got EOFError or IOError -- exiting')
@@ -105,6 +111,7 @@ def worker(conn, inqueue, outqueue, initializer=None, initargs=(), maxtasks=None
 
         job, i, func, args, kwds = task
         try:
+            state['STATE'] = 'RUNNING'
             result = (True, func(*args, **kwds))
         except Exception, e:
             result = (False, e)
@@ -178,7 +185,7 @@ class Pool(object):
             exitpriority=15
             )
     
-    def get_tracebacks(self):
+    def introspect(self):
         ret = []
         for process in self._pool:
             process.intro_conn.send(True)
